//
//  FilmsTableViewController.m
//  Films
//
//  Created by Santiago Pavón on 18/10/13.
//  Copyright (c) 2013 UPM. All rights reserved.
//

#import "FilmsTableViewController.h"
#import "FilmViewController.h"
#import "Film.h"
#import "FilmCell.h"

@interface FilmsTableViewController () <UISearchBarDelegate, UISearchDisplayDelegate>

// Todas las peliculas:
@property (strong,nonatomic) NSArray *filmsArray;

// Resultados de las busquedas:
@property (strong,nonatomic) NSArray *filteredFilmsArray;

@end

@implementation FilmsTableViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    UINib * nib = [UINib nibWithNibName:@"FilmCell" bundle:nil];
    
    [self.searchDisplayController.searchResultsTableView registerNib:nib
                                              forCellReuseIdentifier:@"Film Cell"];
    
    [self.tableView registerNib:nib
         forCellReuseIdentifier:@"Film Cell"];
    
    
    // Todas las peliculas:
    self.filmsArray = @[[[Film alloc] initWithTitle:@"Lluvia de albóndigas"
                                              genre:@"Infantil"],
                        [[Film alloc] initWithTitle:@"Buscando a Nemo"
                                              genre:@"Infantil"],
                        [[Film alloc] initWithTitle:@"Bambi"
                                              genre:@"Infantil"],
                        [[Film alloc] initWithTitle:@"Indiana Jones y el templo maldito"
                                              genre:@"Familiar"],
                        [[Film alloc] initWithTitle:@"Mentiras arriesgadas"
                                              genre:@"Familiar"],
                        [[Film alloc] initWithTitle:@"La Rosa Púrpura del Cairo"
                                              genre:@"Familiar"],
                        [[Film alloc] initWithTitle:@"El juego de Ender"
                                              genre:@"Familiar"],
                        [[Film alloc] initWithTitle:@"Parque Jurásico"
                                              genre:@"Familiar"],
                        [[Film alloc] initWithTitle:@"El silencio de los corderos"
                                              genre:@"Adultos"],
                        [[Film alloc] initWithTitle:@"El exorcista"
                                              genre:@"Adultos"],
                        [[Film alloc] initWithTitle:@"El último tango en París"
                                              genre:@"Adultos"],
                        [[Film alloc] initWithTitle:@"Tiburón"
                                              genre:@"Adultos"]
                        ];
    
    // Resultados de las busquedas:
    self.filteredFilmsArray = [NSArray array];
    

    
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Table view data source

// Return the number of sections.
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

// Return the number of rows in the section.
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Mirar que array debo consultar: el array normal o el de resultados de busquedas:
    if (tableView == self.searchDisplayController.searchResultsTableView) {
        return [self.filteredFilmsArray count];
    } else {
        return [self.filmsArray count];
    }
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellId = @"Film Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellId
                             forIndexPath:indexPath];
    
    Film *film;
    // Mirar que array debo consultar: el array normal o el de resultados de busquedas:
    if (tableView == self.searchDisplayController.searchResultsTableView) {
        film = self.filteredFilmsArray[indexPath.row];
    } else {
        film = self.filmsArray[indexPath.row];
    }
    
    // cell.textLabel.text = film.title;
    // cell.detailTextLabel.text = film.genre;

    ((FilmCell*)cell).titleLabel.text = film.title;
    ((FilmCell*)cell).genreLabel.text = film.genre;

    return cell;
}

/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/

/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
        [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
    }   
}
*/

/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
{
}
*/

/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/


#pragma mark - UISearchDisplayController Delegate Methods


// Ha cambido el texto a buscar.
-(BOOL)    searchDisplayController:(UISearchDisplayController *)controller
  shouldReloadTableForSearchString:(NSString *)searchString
{
    NSString * genre;
    if (self.searchDisplayController.searchBar.showsScopeBar) {
        NSUInteger index = self.searchDisplayController.searchBar.selectedScopeButtonIndex;
        genre = self.searchDisplayController.searchBar.scopeButtonTitles[index];
    }
    [self updateFilteredFilmsWithTitle:searchString
                                 genre:genre];
    return YES;
}


// Ha cambiado el scope a buscar.
-(BOOL)    searchDisplayController:(UISearchDisplayController *)controller
   shouldReloadTableForSearchScope:(NSInteger)searchOption
{
    NSString * genre = self.searchDisplayController.searchBar.scopeButtonTitles[searchOption];
    NSString * title = self.searchDisplayController.searchBar.text;
    
    [self updateFilteredFilmsWithTitle:title
                                 genre:genre];
    return YES;
}


#pragma mark - Film Filtering

-(void) updateFilteredFilmsWithTitle:(NSString*)text
                               genre:(NSString*)genre
{
    // Asigno a self.filteredFilmsArray un array con las peliculas
    // que contengan "text" en su titulo y que sean del genero "genre".
    
    // Acepto genre igual a nil por si no uso scopes.
    
    // Busco usando un NSPredicate.
    // - "SELF.title" indica buscar en la propiedad "title".
    // - “contains[c]” indica contener sin distinguir mayusculas
    //                 de minusculas.
    // - "%@" se sustituye por el texto a buscar.
    NSPredicate *pred = [NSPredicate predicateWithFormat:@"SELF.title contains[c] %@",text];
    
    NSArray *tmp = [self.filmsArray filteredArrayUsingPredicate:pred];
    
    if (genre && ![genre isEqualToString:@"Todas"]) {
        NSPredicate *pred2 = [NSPredicate predicateWithFormat:@"SELF.genre contains[c] %@",genre];
        tmp = [tmp filteredArrayUsingPredicate:pred2];
    }
    
    self.filteredFilmsArray = tmp;
}



#pragma mark - TableView Delegate

-(void) tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    [self performSegueWithIdentifier:@"Show Film"
                              sender:tableView];
}


#pragma mark - Navigation

- (void) prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([segue.identifier isEqualToString:@"Show Film"]) {
        
        FilmViewController *fvc = segue.destinationViewController;
        
        if (sender == self.searchDisplayController.searchResultsTableView) {
            
            NSIndexPath *ip = [self.searchDisplayController.searchResultsTableView indexPathForSelectedRow];
            
            fvc.film = self.filteredFilmsArray[ip.row];
            
        } else {
            
            NSIndexPath *ip = [self.tableView indexPathForSelectedRow];
            
            fvc.film = self.filmsArray[ip.row];
        }
    }
}



@end
